# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2013 E-MIPS (http://www.e-mips.com.ar) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from openerp import models, fields, api, exceptions, _
import openerp.addons.decimal_precision as dp
from openerp.exceptions import ValidationError
from openerp import netsvc

# TODO: Tomar el currency_rate del dia de hoy o de la fecha del despacho

import re
re_simi_number = re.compile('\d{2}\d{3}SIMI\d{2}\d{6}[A-Z]')
#re_simi_number = re.compile('\d{2}\d{3}[A-Z]{2}\d{2}\d{6}[A-Z]')

class import_clearance_expense_line(models.Model):
    _name = 'import.clearance.expense.line'
    _description = 'Clearance Expense Line'

    @api.one
    @api.depends('clearance_line_tax_id', 'price_unit', 'product_id', 'partner_id', 'clearance_id.currency_id')
    def _amount_line(self):
        tax_obj = self.env['account.tax']
        cur_obj = self.env['res.currency']
        price_subtotal = 0.0
        amount_total = 0.0
        taxes = tax_obj.compute_all(self.clearance_line_tax_id, self.price_unit, 1, product=self.product_id, partner=self.partner_id)
        price_subtotal = taxes['total']
        amount_total = taxes['total_included']
        cur = self.clearance_id.currency_id
        self.price_subtotal = cur_obj.round(cur, price_subtotal)
        self.amount_total = cur_obj.round(cur, amount_total)

    # TODO: Agregar columns de date_due y date_invoice
    name = fields.Char('Description', size=64)
    invoice_number = fields.Char('Invoice Ref', size=64, required=True)
    partner_id = fields.Many2one('res.partner', 'Partner', change_default=True, required=True)
    product_id = fields.Many2one('product.product', 'Product')
    clearance_id = fields.Many2one('import.clearance', 'Related Clearance')
    expense_account_id = fields.Many2one('account.account', 'Account', required=True, domain=[('type','<>','view'), ('type', '<>', 'closed')], help="The income or expense account related to the selected product.")
    account_id = fields.Many2one('account.account', 'Account Payable', required=True, help="The partner account used for this invoice.")
    price_unit = fields.Float('Unit Price', required=True, digits_compute=dp.get_precision('Product Price'))
    price_subtotal = fields.Float(compute=_amount_line, string='Subtotal', digits_compute=dp.get_precision('Account'), store=True)
    amount_total = fields.Float(compute=_amount_line, string='Total', digits_compute=dp.get_precision('Account'), store=True)
    clearance_line_tax_id = fields.Many2many('account.tax', 'account_clearance_line_tax', 'clearance_line_id', 'tax_id', 'Taxes', domain=[('parent_id','=',False)])

    def product_id_change(self, cr, uid, ids, product, name='', partner_id=False, price_unit=False, currency_id=False, context=None, company_id=None):

        fpos_obj = self.pool.get('account.fiscal.position')

        if context is None:
            context = {}
        company_id = company_id if company_id != None else context.get('company_id',False)
        context = dict(context)
        context.update({'company_id': company_id, 'force_company': company_id})
        if not partner_id:
            raise exceptions.except_orm(_('No Partner Defined!'),_("You must first select a partner!") )
        if not product:
            if type in ('in_invoice', 'in_refund'):
                return {'value': {}, 'domain':{'product_uom':[]}}
        part = self.pool.get('res.partner').browse(cr, uid, partner_id, context=context)

        result = {}
        result['account_id'] = part.property_account_payable.id
        res = self.pool.get('product.product').browse(cr, uid, product, context=context)

        account_id = res.property_account_expense.id
        if not account_id:
            account_id = res.categ_id.property_account_expense_categ.id

        fpos = part.property_account_position
        account_id = fpos_obj.map_account(cr, uid, fpos, account_id)
        if account_id:
            result['expense_account_id'] = account_id

        taxes = res.supplier_taxes_id and res.supplier_taxes_id or (account_id and self.pool.get('account.account').browse(cr, uid, account_id, context=context).tax_ids or False)

        tax_id = fpos_obj.map_tax(cr, uid, fpos, taxes)

        result.update( {'price_unit': price_unit or res.standard_price,'clearance_line_tax_id': tax_id} )
        result['name'] = res.partner_ref

        #result['uom_id'] = res.uom_id.id
        if res.description:
            result['name'] += '\n'+res.description

        res_final = {'value':result, 'domain': {}}

        if not company_id or not currency_id:
            return res_final

        company = self.pool.get('res.company').browse(cr, uid, company_id, context=context)
        currency = self.pool.get('res.currency').browse(cr, uid, currency_id, context=context)

        if company.currency_id.id != currency.id:
            res_final['value']['price_unit'] = res.standard_price
            new_price = res_final['value']['price_unit'] * currency.rate
            res_final['value']['price_unit'] = new_price

        return res_final

    def onchange_partner_id(self, cr, uid, ids, partner_id, company_id=False):
        acc_id = False

        if partner_id:

            p = self.pool.get('res.partner').browse(cr, uid, partner_id)
            if company_id:
                if (p.property_account_payable.company_id and (p.property_account_payable.company_id.id != company_id)):
                    property_obj = self.pool.get('ir.property')
                    pay_pro_id = property_obj.search(cr,uid,[('name','=','property_account_payable'),('res_id','=','res.partner,'+str(partner_id)+''),('company_id','=',company_id)])
                    if not pay_pro_id:
                        pay_pro_id = property_obj.search(cr,uid,[('name','=','property_account_payable'),('company_id','=',company_id)])
                    pay_line_data = property_obj.read(cr,uid,pay_pro_id,['name','value_reference','res_id'])
                    pay_res_id = pay_line_data and pay_line_data[0].get('value_reference',False) and int(pay_line_data[0]['value_reference'].split(',')[1]) or False
                    if not pay_res_id:
                        raise exceptions.except_orm(_('Configuration Error!'),
                            _('Cannot find a chart of accounts for this company, you should create one.'))
                    account_obj = self.pool.get('account.account')
                    pay_obj_acc = account_obj.browse(cr, uid, [pay_res_id])
                    p.property_account_payable = pay_obj_acc[0]

            acc_id = p.property_account_payable.id

        result = {'value': {
            'account_id': acc_id,
            }
        }

        return result

    def move_line_get_taxes(self, cr, uid, line_taxes, partner_id):
        res = []
        for k, t in line_taxes.iteritems():
            if not t['amount'] \
                    and not t['tax_code_id'] \
                    and not t['tax_amount']:
                continue
            res.append({
                'type':'tax',
                'name':t['name'],
                'partner_id': partner_id,
                'price_unit': t['amount'],
                'quantity': 1,
                'price': t['amount'] or 0.0,
                'account_id': t['account_id'],
                'tax_code_id': t['tax_code_id'],
                'tax_amount': t['tax_amount'],
                'account_analytic_id': t['account_analytic_id'],
            })
        return res

    def move_line_get_item(self, cr, uid, line, partner_id, context=None):
        return {
            'type':'src',
            'name': line.name.split('\n')[0][:64],
            'partner_id': partner_id,
            'price_unit':line.price_unit,
            'quantity': 1,
            'price':line.price_subtotal,
            'account_id':line.expense_account_id.id,
            'product_id':line.product_id.id,
            'taxes':line.clearance_line_tax_id,
        }

    def move_line_get(self, cr, uid, line, partner_id, context=None):
        res = []
        tax_obj = self.pool.get('account.tax')
        cur_obj = self.pool.get('res.currency')
        if context is None:
            context = {}
        inv = line.clearance_id
        company_currency = self.pool['res.company'].browse(cr, uid, inv.company_id.id).currency_id.id

        mres = self.move_line_get_item(cr, uid, line, partner_id, context)
        if not mres:
            res
        res.append(mres)
        tax_code_found= False
        for tax in tax_obj.compute_all(cr, uid, line.clearance_line_tax_id, line.price_unit, 1,
                                       product=line.product_id, partner=line.partner_id)['taxes']:

            tax_code_id = tax['base_code_id']
            tax_amount = line.price_subtotal * tax['base_sign']

            if tax_code_found:
                if not tax_code_id:
                    continue
                res.append(self.move_line_get_item(cr, uid, line, partner_id, context))
                res[-1]['price'] = 0.0
                res[-1]['account_analytic_id'] = False
            elif not tax_code_id:
                continue
            tax_code_found = True

            res[-1]['tax_code_id'] = tax_code_id
            res[-1]['tax_amount'] = cur_obj.compute(cr, uid, inv.currency_id.id, company_currency, tax_amount)#, context={'date': inv.date_invoice})
        return res


class import_clearance(models.Model):
    _name = "import.clearance"
    _description="Import Clearance"
    _inherit = ['mail.thread', 'ir.needaction_mixin']
    _order = "date desc"
    _track = {
        'state': {
            'account_import_clearance.mt_clearance_confirmed':
                lambda self, cr, uid, obj, ctx=None: obj.state in ['confirm'],
            'account_import_clearance.mt_clearance_posted':
                lambda self, cr, uid, obj, ctx=None: obj.state in ['post']
        },
    }

    def _get_currency_usd(self, cr, uid, context=None):

        # Seteamos por defecto dolares en currency_usd_id
        res = self.pool['ir.model.data'].get_object(cr, uid, "base", "USD", context)
        return res.id

    def _get_currency(self, cr, uid, context=None):

        company_id = self.pool.get('res.users')._get_company(cr, uid, context=context)
        if not company_id:
            raise exceptions.Warning(_('There is no default company for the current user!'))

        company = self.pool.get('res.company').browse(cr, uid, company_id, context=context)

        return company.currency_id.id

    def _get_journal(self, cr, uid, context=None):
        if context is None:
            context = {}

        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_id = context.get('company_id', user.company_id.id)
        journal_obj = self.pool.get('account.journal')
        domain = [('company_id', '=', company_id)]

        domain.append(('type', '=', 'purchase'))
        res = journal_obj.search(cr, uid, domain, limit=1)
        return res and res[0] or False

#    @api.one
#    @api.depends('expense_invoice_ids', 'expense_invoice_ids.amount_untaxed', 'invoice_ids', 'invoice_ids.amount_total')
#    def _base_taxable(self):
#        base_taxable_sum = 0.0
#        base_taxable_sum += sum([x.amount_untaxed for x in self.expense_invoice_ids if x.clearance_taxable is True])
#        base_taxable_sum += sum([x.amount_total for x in self.invoice_ids])
#        self.base_taxable = base_taxable_sum

    @api.one
    @api.depends('expense_invoice_ids', 'expense_invoice_ids.amount_total', 'additional_invoice_ids', 'additional_invoice_ids.price_subtotal')
    def _expense_sum(self):
        expense_sum = 0.0

        invoices_amount = self.expense_invoice_ids.get_invoice_amounts_in_company_cur()
        expense_sum += sum(map(lambda x: x['amount_total'], invoices_amount.values()))

        # TODO: Sumar las aditional invoice que son lines pero siempre en ARS.
        # Aca se intenta hacerlo por medio de la currency_rate
        for add_invoice_line in self.additional_invoice_ids:
            invoice = add_invoice_line.invoice_id
            if invoice.is_multi_currency and \
                    not invoice.currency_converted:
                currency_rate = invoice.currency_rate
            else:
                currency_rate = 1.0

            expense_sum += add_invoice_line.price_subtotal * currency_rate

        self.expense = expense_sum
        return

    @api.one
    @api.depends('expense_invoice_ids', 'expense_invoice_ids.amount_total', 'additional_invoice_ids', 'additional_invoice_ids.price_subtotal', 'insurance', 'freight', 'fob')
    def _expense_percent(self):
        expense_amount = 0.0
        fob = self.fob * self.currency_rate
        expense_amount = self.expense + (self.freight+self.insurance) * self.currency_rate

        if fob != 0.0:
            self.expense_percent = (expense_amount / fob) * 100

    @api.one
    @api.depends('advancement_detail_ids', 'advancement_detail_ids.amount_total', 'payable_grouped_ids', 'payable_grouped_ids.amount_due', 'fob')
    def _clearance_balance(self):
        clearance_balance = 0.0
        # Pagos de adelantos
        for advancement in self.advancement_detail_ids:
            clearance_balance += advancement.amount_total
        for payable_group in self.payable_grouped_ids:
            clearance_balance -= payable_group.amount_due

        if self.fob != 0.0:
            self.clearance_balance = clearance_balance

    @api.onchange('currency_id')
    def onchange_currency_id(self):
        currency_obj = self.env['res.currency']
        if not self.currency_id:
            self.is_multi_currency = False
            self.currency_rate = 1
        else:
            currency_rate = self.currency_id.calc_rate_for_date(date=self.date)
            #self.currency_rate = 1.0 / currency_rate
            self.currency_rate = currency_rate

            if currency_rate == 1:
                self.is_multi_currency = False
            else:
                self.is_multi_currency = True

    @api.onchange('fob', 'fob_usd', 'currency_rate_usd')
    def onchange_fobs(self):
        currency_obj = self.env['res.currency']
        rate_usd = self.currency_rate_usd

        if not rate_usd or not self.fob_usd:
            return

        fob_in_company_cur = self.fob_usd / rate_usd

        # TODO: Hacer readonly el currency_rate
        self.currency_rate = (self.fob_usd*rate_usd) / self.fob  # / fob_in_company_cur

    @api.constrains('simi')
    def _check_simi_number(self):
        for clearance in self:
            simi = clearance.simi
            if not simi or simi == '' or re_simi_number.match(simi):
                return True

        raise ValidationError(_("SIMI Number does not match format"))

    @api.onchange('simi')
    def change_simi_number(self):
        # Si de borra totalmente, borramos los demas campos de fecha
        if not self.simi:
            self.simi_request_date = False
            self.simi_approved_date = False
            self.simi_due_date = False
            self.simi_cancel_date = False
            self.simi_amount = 0.0

    @api.one
    @api.depends('simi_request_date', 'simi_approved_date',
                 'simi_cancel_date')
    def _simi_state(self):

        simi_state = 'to request'

        # Chequemos las fechas
        if self.simi_cancel_date:
            simi_state = 'cancel'
        elif self.simi_approved_date:
            simi_state = 'approved'
        elif self.simi_request_date:
            simi_state = 'request'

        self.simi_state = simi_state

    # TODO: Agregar los readonly por state a los campos
    name = fields.Char('Clearance', size=64, readonly=False,
            states={'closed':[('readonly', True)]})
    origin = fields.Char('Source Document', size=32, readonly=True,
            states={'draft':[('readonly', False)]})
    date = fields.Datetime('Clearance Date', readonly=True,
            states={'draft':[('readonly', False)]})
    bcra_pending = fields.Boolean('Pending BCRA', states={'closed':[('readonly',True)]})
    insurance = fields.Float('Insurance', digits=(16, 2), required=False, help='Insurance')
    freight = fields.Float('Freight', digits=(16, 2), required=False, help='Freight')
    period_id = fields.Many2one('account.period', 'Force Period',
            domain=[('state','<>','done')], help="Keep empty to use the period of the " \
            "clearance date.", readonly=True,
            states={'draft':[('readonly',False)], 'confirm':[('readonly',False)]})
    date_arrival = fields.Datetime('Arrival Date', readonly=True,
            states={'draft':[('readonly', False)]})
    currency_id = fields.Many2one('res.currency', 'Currency', required=True,
            readonly=True, states={'draft':[('readonly', False)],
                                   'confirm':[('readonly', False)]},
            track_visibility='onchange')
    company_currency_id = fields.Many2one('res.currency', 'Company Currency', required=True,
            states={'draft':[('readonly', False)]})
    currency_usd_id = fields.Many2one('res.currency', 'Currency (USD)', required=True,
            states={'draft':[('readonly', False)]})
    is_multi_currency = fields.Boolean('Is Multicurrency')
    currency_rate_usd = fields.Float('Currency Rate USD', readonly=True,
            states={'draft':[('readonly', False)],
                    'confirm':[('readonly', False)]}, track_visibility='onchange')
    currency_rate = fields.Float('Currency Rate', readonly=True,
            states={'draft':[('readonly', False)],
                    'confirm':[('readonly', False)]}, track_visibility='onchange')
    journal_id = fields.Many2one('account.journal', 'Journal', required=True,
            readonly=True, states={'draft':[('readonly', False)]})
    broker_id = fields.Many2one('res.partner', 'Broker', readonly=True,
            states={'draft': [('readonly', False)], 'confirm': [('readonly', False)]})
    partner_id = fields.Many2one('res.partner', 'Supplier', readonly=True,
            states={'draft': [('readonly', False)], 'confirm': [('readonly', False)]})
    invoice_ids = fields.Many2one('account.invoice', 'Supplier Invoices',
            readonly=False, states={'closed': [('readonly', True)],
                                    'cancel': [('readonly', True)]})
    # TODO: Si las borro desde el Clearance, las tengo que borrar fisicamente tambien
    expense_invoice_ids = fields.One2many('account.invoice', 'clearance_id',
            'Expense Invoices', readonly=True, states={'draft':[('readonly', False)],
                                                       'confirm':[('readonly', False)]})
    additional_invoice_ids = fields.One2many('account.invoice.line', 'clearance_id',
                                             'Additional Invoices')
    account_move_ids = fields.Many2many('account.move', 'clearance_move_rel',
            'clearance_id', 'move_id', 'All moves', readonly=True,
            help="All the account moves associated with this clearance")
    advancement_detail_ids = fields.One2many('import.advancement.detail',
            'clearance_id', 'Advancement Details', readonly=True,
            states={'draft':[('readonly', False)]})
    payable_grouped_ids = fields.One2many('clearance.grouped.payable',
            'clearance_id', 'Payable Grouped Partners', readonly=True,
            states={'post':[('readonly', False)]})
    picking_id = fields.Many2one('stock.picking', 'Incoming Shipment', readonly=True)
    purchase_id = fields.Many2one('purchase.order', 'Purchase Order', readonly=True,
            states={'draft':[('readonly', False)]})
    description = fields.Text('Description')
    company_id = fields.Many2one('res.company', 'Company', required=True,
            readonly=True, states={'draft':[('readonly', False)]})
    # SIMI Management
    simi = fields.Char('SIMI', size=32, readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_request_date = fields.Date('SIMI Request Date', readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_approved_date = fields.Date('SIMI Approved Date', readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_cancel_date = fields.Date('SIMI Cancelled Date', readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_due_date = fields.Date('SIMI Due Date', readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_amount = fields.Float(string='SIMI Amount',
            digits_compute=dp.get_precision('Account'), readonly=False,
            states={'closed': [('readonly', True)]}, track_visibility='onchange')
    simi_state = fields.Selection([
        ('to request','To Request'),
        ('request','Requested'),
        ('approved','Approbed'),
        ('cancel','Cancelled'), ],'SIMI State', compute=_simi_state,
        store=True, select=True, readonly=True, track_visibility='onchange')
    #'expense_move_id': fields.many2one('account.move', 'Expense Entry', readonly=True, select=1, help="Link to the automatically generated Journal Items."),
    base_taxable = fields.Float(string='Base Taxable',
            digits_compute=dp.get_precision('Account'), track_visibility='onchange')
    fob = fields.Float(string='FOB', digits_compute=dp.get_precision('Account'),
            track_visibility='onchange')
    fob_usd = fields.Float(string='FOB in USD', digits_compute=dp.get_precision('Account'),
            track_visibility='onchange')
    expense = fields.Float(compute=_expense_sum, string='Expense',
            digits_compute=dp.get_precision('Account'))
    expense_percent = fields.Float(compute=_expense_percent, string='Expense Percent',
            digits_compute=dp.get_precision('Account'))
    clearance_balance = fields.Float(compute=_clearance_balance, string='Clearance Balance',
            digits_compute=dp.get_precision('Account'))
    state = fields.Selection([
        ('draft','Draft'),
        ('confirm','Confirmed'),
        ('post','Posted'),
        ('closed','Closed'),
        ('cancel','Cancelled'), ],'Status', select=True,
        readonly=True, track_visibility='onchange')

    _defaults = {
        'state': 'draft',
        'date': lambda *args: time.strftime('%Y-%m-%d %H:%M:%S'),
        'journal_id': _get_journal,
        'currency_id': _get_currency,
        'company_currency_id': _get_currency,
        'currency_usd_id': _get_currency_usd,
        'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'purchase.requisition', context=c),
    }

    def name_get(self, cr, uid, ids, context=None):
        if isinstance(ids, (list, tuple)) and not len(ids):
            return []
        if isinstance(ids, (long, int)):
            ids = [ids]

        reads = self.read(cr, uid, ids, ['name','origin','state'], context=context)
        res = []
        for record in reads:
            name = (record['name'] and record['name'] or '') + (record['origin'] and '[' + record['origin'] + ']' or '')
            res.append((record['id'], name))

        return res

#    def write(self, cr, uid, ids, vals, context=None):
#
#        res = super(import_clearance, self).write(
#            cr, uid, ids,  vals, context=context)
#
#        if 'invoice_ids' in vals:
#            import ipdb; ipdb.set_trace()
#        return res

#    def onchange_journal_id(self, cr, uid, ids, journal_id=False, context=None):
#        result = {}
#        if journal_id:
#            journal = self.pool.get('account.journal').browse(cr, uid, journal_id, context=context)
#            currency_id = journal.currency and journal.currency.id or journal.company_id.currency_id.id
#            company_id = journal.company_id.id
#            result = {'value': {
#                    'currency_id': currency_id,
#                    'company_id': company_id,
#                    }
#                }
#        return result

    def action_get_expense_move(self, cr, uid, ids, context=None):

        piml = []
        for clearance in self.browse(cr, uid, ids, context=context):
            for inv in clearance.expense_invoice_ids:
                for iml in inv.move_id.line_id:
                    if iml.account_id.type == 'payable':
                        iml.advancement_type_id = inv.advancement_type_id
                        piml.append(iml)

        return piml

    def action_create_advancement(self, cr, uid, ids, context=None):
        advancement_detail_obj = self.pool.get("import.advancement.detail")
        period_obj = self.pool.get('account.period')

        def _filter_new(advancement):
            return advancement.state == 'new'

        for clearance in self.browse(cr, uid, ids, context=context):
            clearance_moves = []
            # Si no tenemos el periodo seteado, lo forzamos
            if not clearance.period_id:
                period_ids = period_obj.find(cr, uid, clearance.date, context=context)
                period_id = period_ids and period_ids[0] or False
                self.write(cr, uid, clearance.id, {'period_id': period_id}, context=context)

            # Creamos el movimiento de adelantos
            adv_details = clearance.advancement_detail_ids.filtered(_filter_new)
            advancement_detail_ids = adv_details.ids

            move0_id = advancement_detail_obj.create_advancement_move(cr, uid, clearance,
                                                                      advancement_detail_ids,
                                                                      context=context)

            clearance_vals = {}

            # Modificamos el clearance, le agregamos el move de adelanto
            if move0_id:
                clearance_moves.append(move0_id)
                clearance_move_ids = [(4, move_id, _) for move_id in clearance_moves]
                clearance_vals['account_move_ids'] = clearance_move_ids
                self.write(cr, uid, [clearance.id], clearance_vals, context=context)

        return True

    def action_confirm(self, cr, uid, ids, context=None):
        for clearance in self.browse(cr, uid, ids, context=context):
            clearance_vals = {
                'state': 'confirm'
            }

            if len(clearance.advancement_detail_ids) == 0:
                raise exceptions.except_orm(_("Fill information"), _("At least one advancement should be loaded"))

            for adv in clearance.advancement_detail_ids:
                if adv.state == 'new':
                    raise exceptions.except_orm(_("Advancement Error"), _("All advancement shall be published."))

            self.write(cr, uid, [clearance.id], clearance_vals, context=context)

        return True

    def action_post(self, cr, uid, ids, context=None):
        clearance_payable_obj = self.pool.get("clearance.grouped.payable")
        wf_service = netsvc.LocalService("workflow")

        # Obtenemos los movimientos de gastos
        clearance = self.browse(cr, uid, ids[0], context=context)

#        # Chequeamos que se haya cargado al menos un adelanto
#        # NOTE: Ahora se chequea al confirmar
        adv_details = clearance.advancement_detail_ids

#        if not len(adv_details):
#            raise exceptions.except_orm(_("Error Posting!"), _("You have to define at least one Advancement Detail to post this clearance"))

        if not len(clearance.expense_invoice_ids):
            raise exceptions.except_orm(_("Error Posting!"), _("There are no expense " \
                    "invoices in this clearance."))

        clearance_moves = []
        for inv in clearance.expense_invoice_ids:
            if inv.state == 'draft':
                wf_service.trg_validate(uid, 'account.invoice', inv.id, 'invoice_open', cr)
                inv.refresh()
                clearance_moves.append(inv.move_id.id)

        piml = self.action_get_expense_move(cr, uid, ids, context=context)

#        # Creamos el movimiento de adelantos TODO: BORRAR
#        advancement_detail_ids = map(lambda x: x.id, adv_details)
#        move0_id = advancement_detail_obj.create_advancement_move(cr, uid,
#                ids[0], advancement_detail_ids, context=context)
#        clearance_moves.append(move0_id)

        # Creamos las deudas agrupadas del Despacho
        for iml in piml:

            vals = {
                'name': iml.name,
                'account_id': iml.account_id.id,
                'partner_id': iml.partner_id.id,
                'amount_due': iml.credit,
                'advancement_type_id': iml.advancement_type_id.id,
                'clearance_id': ids[0]
            }

            clearance_payable_obj.create(cr, uid, vals, context=context)

            # Modificamos el clearance, le agregamos el move de adelanto
            clearance_move_ids = [(4, move_id, _) for move_id in clearance_moves]
            clearance_vals = {
                'account_move_ids': clearance_move_ids,
                'state': 'post'
            }
            self.write(cr, uid, [clearance.id], clearance_vals, context=context)

        return True

    def action_pay_and_close(self, cr, uid, ids, context=None):
        advancement_detail_obj = self.pool.get("import.advancement.detail")
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get("account.move.line")

        # Creamos el asiento contable que concilia con todos los gastos
        for clearance in self.browse(cr, uid, ids, context=context):
            move = {
                'ref': clearance.name + '[' + clearance.origin +']',
                'journal_id': clearance.journal_id.id,
                'date': clearance.date, # TODO: Agregar campo fecha
                #'narration': inv.comment, # Agregar una narration acorde
                'company_id': clearance.company_id.id,
            }

            move_id = move_obj.create(cr, uid, move, context=context)
            move_lines_to_reconcile = {}
            advancement_grouped = {}

            # Agregamos el account_move de la factura del proveedor
            account_move_ids = []
            if not len(clearance.invoice_ids):
                raise exceptions.except_orm(_('Supplier Invoice Error!'),_("At least one supplier invoice should be associated with this clearance!") )

            # TODO: Hacerlo parametrizable. Por ahora, se comenta.
#            for invoice in clearance.invoice_ids:
#                if invoice.state not in ('open', 'paid'):
#                    raise exceptions.except_orm(_('Supplier Invoice Error!'),_("Supplier Invoice should be validated!") )
#
                # TODO: Agregar este linkeo desde la validacion de la factura
#                account_move_ids.append((4, invoice.move_id.id, _))

            for payable in clearance.payable_grouped_ids:
                if not payable.advancement_type_id:
                    raise exceptions.except_orm(_("Payment Error!"), _("You have to define which payment type is used for each Partner. Please, fill the column Type in Grouped Payables"))

                advancement_type = payable.advancement_type_id

                # Linea del payable
                move_line = {
                    'name': payable.name,
                    'debit': payable.amount_due,
                    'partner_id': payable.partner_id.id,
                    'credit': 0.0,
                    'account_id': payable.account_id.id,
                    'journal_id': clearance.journal_id.id,
                    'period_id': clearance.period_id.id,
                    'date': clearance.date,
                    'move_id': move_id,
                }
                payable_line_id = move_line_obj.create(cr, uid, move_line, context=context)

                for inv in clearance.expense_invoice_ids:
                    for iml in inv.move_id.line_id:
                        if iml.partner_id.id == move_line['partner_id'] and \
                            iml.account_id.id == move_line['account_id']:
                                #print move_line['debit'], iml.credit
                                move_lines_to_reconcile[(iml.partner_id.id, iml.account_id.id)] = [payable_line_id, iml.id]


                # Linea del adelanto
                move_line = {
                    'name': payable.name,
                    'debit': 0.0,
                    'credit': payable.amount_due,
                    'account_id': advancement_type.account_id.id,
                    'journal_id': clearance.journal_id.id,
                    'period_id': clearance.period_id.id,
                    'date': clearance.date,
                    'move_id': move_id,
                }
                move_line_obj.create(cr, uid, move_line, context=context)

                # TODO: Borrar. Esta porcion comentada no tiene mucho sentido
#                # Chequeamos el advancement_detail para saber si tenemos que dejar un saldo o lo marcamos como hechos
#                advancement_detail_obj.write(cr, uid, advancement_detail.id, {'state':'post'}, context=context)
#                if advancement_detail in advancement_grouped:
#                    advancement_grouped[advancement_detail] += payable.amount_due
#                else:
#                    advancement_grouped[advancement_detail] = payable.amount_due

            # Posteamos el account.move
            move_obj.post(cr, uid, [move_id], context=context)

            for rec_ids in move_lines_to_reconcile.values():
                reconcile = move_line_obj.reconcile(cr, uid, rec_ids, writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context=context)

            account_move_ids.append((4, move_id, _))
            clearance_vals = {
                'account_move_ids': account_move_ids,
                'state': 'closed'
            }
            self.write(cr, uid, [clearance.id], clearance_vals, context=context)
        return True

#    def get_balance_advancement(self, cr, uid, ids, context=None):
#        advancement_detail_obj = self.pool.get("import.advancement.detail")
#        adv_ids = advancement_detail_obj.search(cr, uid, [('state','=','balance'),('clearance_id','=',False)], context=context)
#        adv = [(4, aid) for aid in adv_ids]
#        clearance_id = ids[0]
#
#        # Agregamos los saldos a este despacho
#        self.write(cr, uid, clearance_id, {'advancement_detail_ids': adv}, context=context)
#        return True

    def action_cancel(self, cr, uid, ids, context=None):
        #reconcile_pool = self.pool.get('account.move.reconcile')
        move_obj = self.pool.get('account.move')
        advancement_detail_obj = self.pool.get("import.advancement.detail")

        #TODO : Esta funcion todavia no anda en todos los estados
        # En confirmado funciona
        # TODO: Chequeo del estado para que no se aprete en cualquier estado
        for clearance in self.browse(cr, uid, ids, context=context):

            move_ids = map(lambda x: x.move_id.id, clearance.advancement_detail_ids)
            move_ids = filter(lambda x: x, move_ids)
            move_ids = list(set(move_ids))

            # Intentamos cancelar y borrar los movimientos de adelantos
            move_obj.button_cancel(cr, uid, move_ids, context=context)
            move_obj.unlink(cr, uid, move_ids, context=context)

            # Borramos los adelantos
            adv_ids = map(lambda x: x.id, clearance.advancement_detail_ids)
            advancement_detail_obj.unlink(cr, uid, adv_ids, context=context)

        res = {
            'state':'cancel',
            #'expense_move_id':False,
        }
        self.write(cr, uid, ids, res)
        return True
#            # refresh to make sure you don't unlink an already removed move
#            # Borramos los asientos de adelantos y los adelantos
#
##            recs = []
##            for line in clearance.move_line_ids:
##                if line.reconcile_id:
##                    recs += [line.reconcile_id.id]
##                if line.reconcile_partial_id:
##                    recs += [line.reconcile_partial_id.id]
##
##            reconcile_pool.unlink(cr, uid, recs)
#
##            if clearance.expense_move_id:
##                move_pool.button_cancel(cr, uid, [clearance.expense_move_id.id])
##                move_pool.unlink(cr, uid, [clearance.expense_move_id.id])
#
#
#            # Borramos los clearance_grouped_payable
#            payables_to_unlink = [p.id for p in clearance.payable_grouped_ids]
#            self.pool.get('clearance.grouped.payable').unlink(cr, uid, payables_to_unlink, context=context)
#
#        res = {
#            'state':'cancel',
#            #'expense_move_id':False,
#        }
#        self.write(cr, uid, ids, res)
#        return True

    def action_cancel_draft(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state':'draft'})
        return True

    def _prepare_invoice(self, cr, uid, clearance, partner, context=None):
        """Prepare the dict of values to create the new invoice"""
        if context is None:
            context = {}
        journal_ids = self.pool.get('account.journal').search(cr, uid,
            [('type', '=', 'purchase'), ('company_id', '=', clearance.company_id.id)],
            limit=1)
        if not journal_ids:
            raise exceptions.except_orm(_('Error!'),
                _('Please define purchase journal for this company: "%s" (id:%d).') % (clearance.company_id.name, clearance.company_id.id))
        invoice_vals = {
            'name': clearance.origin or '',
            'origin': clearance.name,
            'type': 'in_invoice',
            'reference': clearance.name,
            'account_id': partner.property_account_receivable.id,
            'partner_id': partner.id,
            'journal_id': journal_ids[0],
            #'invoice_line': [(6, 0, lines)],
            #'currency_id': clearance.pricelist_id.currency_id.id,
            #'comment': clearance.note,
            #'payment_term': order.payment_term and order.payment_term.id or False,
            'fiscal_position': partner.property_account_position.id,
            'date_invoice': context.get('date_invoice', False),
            'company_id': clearance.company_id.id,
            'clearance_id': clearance.id,
            #'user_id': order.user_id and order.user_id.id or False
        }

        # Care for deprecated _inv_get() hook - FIXME: to be removed after 6.1
        #invoice_vals.update(self._inv_get(cr, uid, clearance, context=context))
        return invoice_vals

    def make_new_invoice(self, cr, uid, ids, partner_id, context=None):
        inv_obj = self.pool.get('account.invoice')
        partner_obj = self.pool.get('res.partner')

        if context is None:
            context = {}

        partner = partner_obj.browse(cr, uid, partner_id, context=context)
        for clearance in self.browse(cr, uid, ids, context=context):

            inv = self._prepare_invoice(cr, uid, clearance, partner, context=context)
            inv_id = inv_obj.create(cr, uid, inv, context=context)
            #data = inv_obj.onchange_payment_term_date_invoice(cr, uid, [inv_id], inv['payment_term'], time.strftime(DEFAULT_SERVER_DATE_FORMAT))
            #if data.get('value', False):
                #inv_obj.write(cr, uid, [inv_id], data['value'], context=context)
            #inv_obj.button_compute(cr, uid, [inv_id])

        return inv_id

import_clearance()

# Tipos de Adelantos
class import_advancement_type(models.Model):
    _name = "import.advancement.type"
    _description="Import Advancement Type"

    name = fields.Char('Advancement Type', size=64)
    account_id = fields.Many2one('account.account', 'Account', required=True)
    partner_id = fields.Many2one('res.partner', 'Partner', required=True)
    show_always = fields.Boolean('Show Always', help="Check this if this type has not to be filtered by partner/broker")

import_advancement_type()


# Adelantos
class import_advancement_detail(models.Model):
    _name = "import.advancement.detail"
    _description = "Import Advancement Type"

    @api.returns("res.currency")
    def _get_default_currency(self):
        return self.env.user.company_id.currency_id

    name = fields.Char('Description', size=64, compute='onchange_advancement_type', store=True)
    advancement_type_id = fields.Many2one('import.advancement.type', 'Type', required=True)
    clearance_id = fields.Many2one('import.clearance', 'Clearance', ondelete="set null")
    # TODO: Hacerlo multimoneda
    #currency_id = fields.Many2one('res.currency', 'Currency', compute='onchange_advancement_type', store=True)
    amount_total = fields.Float(digits_compute=dp.get_precision('Account'), string='Advanced Amount', store=True)
    account_id = fields.Many2one('account.account', 'Advancement Account')
    move_id = fields.Many2one('account.move', string="Move")
    advancement_date = fields.Date(string='Date')
    currency_id = fields.Many2one('res.currency', string='Currency', default=_get_default_currency)
    currency_rate = fields.Float(string='Currency Rate', digits=dp.get_precision('Account'),
                                 default=1)
    state = fields.Selection([('new', 'New'), ('post', 'Posted'), ('paid', 'Paid')], "Status", default='new')

    @api.onchange("currency_id", "advancement_date")
    def onchange_currency_id_date(self):
        if self.currency_id:
            currency_rate = self.currency_id.calc_rate_for_date(date=self.advancement_date)
            #self.currency_rate = 1.0 / currency_rate
            self.currency_rate = currency_rate
        else:
            self.currency_rate = 1

    @api.one
    @api.depends('advancement_type_id')
    def onchange_advancement_type(self):
        if not self.advancement_type_id:
            return None

        #self.currency_id = self.clearance_id.currency_id.id
        self.name = self.advancement_type_id.name
        self.account_id = self.advancement_type_id.account_id

    def create_advancement_move(self, cr, uid, clearance, adv_detail_ids, context=None):
        move_obj = self.env['account.move']

        move_lines = []
        date_today = fields.Date.today()

        journal_id = clearance.journal_id.id
        period_id = clearance.period_id.id

        company_currency = self.env.user.company_id.currency_id

        advancement_details = self.browse(cr, uid, adv_detail_ids, context=context)
        for detail in advancement_details:
            partner_id = detail.advancement_type_id.partner_id

            # Si esta en cero no hacemos asiento
            if detail.amount_total == 0.0:
                self.write(cr, uid, adv_detail_ids, {'state': 'post'}, context=context)
                continue

            name = detail.name or detail.advancement_type_id.name

            # Creamos la linea contable perteneciente al adelanto
            debit_move_line = {
                'name': name,
                'credit': 0.0,
                'account_id': detail.account_id.id,
                'journal_id': journal_id,
                'period_id': period_id,
                'partner_id': partner_id.id,
                'date': date_today,
            }

            # Creamos el asiento a pagar
            credit_move_line = {
                'name': detail.name,
                'debit': 0.0,
                'account_id': partner_id.property_account_payable.id,
                'journal_id': journal_id,
                'period_id': period_id,
                'partner_id': partner_id.id,
                'date': date_today,
                #'date': clearance['date'],
            }

            currency_rate = 0
            amount_currency = 0
            current_currency = detail.currency_id
            debit = credit = detail.amount_total
            currency_id = False
            if company_currency != current_currency:
                # it's a multicurrency advancement, so we compute the corresponding amounts and
                # currency_id
                currency_rate = detail.currency_rate
                amount_currency = debit
                debit *= currency_rate
                credit *= currency_rate
                currency_id = current_currency.id

            debit_move_line.update({"debit": debit, "amount_currency": amount_currency,
                                    "currency_id": currency_id, "currency_rate": currency_rate})
            credit_move_line.update({"credit": credit, "amount_currency": -amount_currency,
                                     "currency_id": currency_id, "currency_rate": currency_rate})

            move_lines.extend([(0, 0, debit_move_line), (0, 0, credit_move_line)])

        # Creamos el movimiento contable
        if not move_lines:
            return None

        clearance_name = clearance.name or ''
        clearance_origin = clearance.origin or ''
        clearance_name = ' '.join([clearance_name, clearance_origin]).strip()
        move = {
            'ref': _("Advancement for ") + ' ' + clearance_name,
            'line_id': move_lines,
            'journal_id': journal_id,
            'date': date_today,
            #clearance['date'],
            'company_id': clearance.company_id.id,
        }

        move_id = move_obj.create(cr, uid, move, context=context)
        move_obj.post(cr, uid, [move_id], context=context)
        self.write(cr, uid, adv_detail_ids, {'state': 'post', 'move_id': move_id}, context=context)
        return move_id


class clearance_grouped_payable(models.Model):
    _name = "clearance.grouped.payable"
    _description = "Clearance Grouped Payable"

    name = fields.Char('Description', size=64)
    clearance_id = fields.Many2one('import.clearance', 'Clearance')
    account_id = fields.Many2one('account.account', 'Payable Account')
    partner_id = fields.Many2one('res.partner', 'Partner', required=True)
    # 'advancement_type_id': fields.many2one('import.advancement.type', 'Type'),
    advancement_type_id = fields.Many2one('import.advancement.type', 'Payment Type')
    amount_due = fields.Float('Amount Due', digits=(16, 2), required=True, help='Amount to pay to this Partner for this clearance')

clearance_grouped_payable()


class account_move_line(models.Model):
    _name = "account.move.line"
    _inherit = "account.move.line"

    def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context=None):

        reconcile_obj = self.pool.get('account.move.reconcile')
        clearance_obj = self.pool.get('import.clearance')
        adv_detail_obj = self.pool.get('import.advancement.detail')

        r_id = super(account_move_line, self).reconcile(
            cr, uid, ids, type='auto',
            writeoff_acc_id=writeoff_acc_id,
            writeoff_period_id=writeoff_period_id,
            writeoff_journal_id=writeoff_journal_id, context=None)

        # Si esta reconciliado, vemos de marcar como pagado el adelanto
        # TODO: Cuando se desconcilia, tenemos que desmarcar ese pagado,
        # o restringir la desconciliación si ya se utilizo. Hay una funcion
        # en account/account_move_line.py para la desconciliacion
        reconcile = reconcile_obj.browse(cr, uid, r_id, context)

        move_ids = map(lambda x: x.move_id.id, reconcile.line_id)
        if not move_ids:
            # TODO: Meter logger warning o algo
            return r_id

        # Buscamos un adelanto de importacion que pueda tener el move de esta conciliacion
        adv_detail_ids = adv_detail_obj.search(cr, uid, [('move_id', 'in', move_ids)], context=context)

        if not adv_detail_ids:
            return r_id

        adv_detail_obj.write(cr, uid, adv_detail_ids, {'state': 'paid'})
        adv_detail = adv_detail_obj.browse(cr, uid, adv_detail_ids[0], context=context)

        clearance = adv_detail.clearance_id

        # Guardamos el asiento de pago en el clearance
        clearance_move_ids = [(4, move_id, _) for move_id in move_ids]
        clearance_vals = {'account_move_ids': clearance_move_ids}
        clearance_obj.write(cr, uid, [clearance.id], clearance_vals, context=context)
        return r_id
