# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 Eynes (http://www.eynes.com.ar)
#    Copyright (c) 2014 E-MIPS (http://www.e-mips.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import models, fields, api, exceptions, _


class clearance_expense_template(models.Model):
    _name = 'clearance.expense.template'
    _description = 'Clearance Expense Template'

    name = fields.Char('Description', size=64, required=True)
    broker_id = fields.Many2one('res.partner', 'Broker')
    expense_line_ids = fields.One2many('clearance.expense.template.line', 'template_id', 'Template Lines')
    company_id = fields.Many2one('res.company', 'Company', required=True)

    _defaults = {
        'company_id': lambda self, cr, uid, c: self.pool.get('res.company')._company_default_get(cr, uid, 'purchase.requisition', context=c),
    }

clearance_expense_template()


class clearance_expense_template_line(models.Model):
    _name = 'clearance.expense.template.line'
    _description = 'Clearance Expense Template Line'

    name = fields.Char('Description', size=64)
    partner_id = fields.Many2one('res.partner', 'Partner', change_default=True, required=True)
    product_id = fields.Many2one('product.product', 'Product')
    template_id = fields.Many2one('clearance.expense.template', 'Template')
    expense_account_id = fields.Many2one('account.account', 'Account', required=True, domain=[('type','<>','view'), ('type', '<>', 'closed')], help="The income or expense account related to the selected product.")
    account_id = fields.Many2one('account.account', 'Account Payable', required=True, help="The partner account used for this invoice.")
    advancement_type_id = fields.Many2one('import.advancement.type', 'Advancement Type', required=False, help="Default advancement type used to pay.")
    clearance_taxable = fields.Boolean('Clearance Taxable', help="Check this if it will be used for base taxable base.")
    in_clearance_currency = fields.Boolean('In Clearance Currency', help="Check this if the invoice will be in clearance currency .")

    def product_id_change(self, cr, uid, ids, product, partner_id=False, context=None, company_id=None):
        if not product:
            return {}
        fpos_obj = self.pool.get('account.fiscal.position')

        if context is None:
            context = {}
        company_id = company_id if company_id != None else context.get('company_id',False)
        context = dict(context)
        context.update({'company_id': company_id, 'force_company': company_id})
        if not partner_id:
            raise exceptions.except_orm(_('No Partner Defined!'),_("You must first select a partner!") )
        part = self.pool.get('res.partner').browse(cr, uid, partner_id, context=context)

        result = {}
        res = self.pool.get('product.product').browse(cr, uid, product, context=context)

        account_id = res.property_account_expense.id
        if not account_id:
            account_id = res.categ_id.property_account_expense_categ.id

        fpos = part.property_account_position
        account_id = fpos_obj.map_account(cr, uid, fpos, account_id)
        if account_id:
            result['expense_account_id'] = account_id

#        result['name'] = res.partner_ref
#
#        if res.description:
#            result['name'] += '\n'+res.description

        res_final = {'value':result, 'domain': {}}

        return res_final

    def onchange_partner_id(self, cr, uid, ids, partner_id, company_id=False):
        acc_id = False

        if partner_id:

            p = self.pool.get('res.partner').browse(cr, uid, partner_id)
            if company_id:
                if (p.property_account_payable.company_id and (p.property_account_payable.company_id.id != company_id)):
                    property_obj = self.pool.get('ir.property')
                    pay_pro_id = property_obj.search(cr,uid,[('name','=','property_account_payable'),('res_id','=','res.partner,'+str(partner_id)+''),('company_id','=',company_id)])
                    if not pay_pro_id:
                        pay_pro_id = property_obj.search(cr,uid,[('name','=','property_account_payable'),('company_id','=',company_id)])
                    pay_line_data = property_obj.read(cr,uid,pay_pro_id,['name','value_reference','res_id'])
                    pay_res_id = pay_line_data and pay_line_data[0].get('value_reference',False) and int(pay_line_data[0]['value_reference'].split(',')[1]) or False
                    if not pay_res_id:
                        raise exceptions.except_orm(_('Configuration Error!'),
                            _('Cannot find a chart of accounts for this company, you should create one.'))
                    account_obj = self.pool.get('account.account')
                    pay_obj_acc = account_obj.browse(cr, uid, [pay_res_id])
                    p.property_account_payable = pay_obj_acc[0]

            acc_id = p.property_account_payable.id

        result = {'value': {
            'account_id': acc_id,
            }
        }

        return result

clearance_expense_template_line()
