# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 E-MIPS/Eynes
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import models, fields, api, exceptions, _

# TODO: Generar una manera de marcar como embarcado el picking. Una posibilidad es un estado nuevo luego de assigned solo para Embarques
# TODO: Migrar pago adelantado
# Chequear el tema multimoneda

class purchase_order(models.Model):
    _name = 'purchase.order'
    _inherit = 'purchase.order'

    clearance_ids = fields.One2many('import.clearance', 'purchase_id', 'Clearances')
    local = fields.Boolean('Local transaction', default=True, compute='_compute_local')

    @api.one
    @api.depends('partner_id', 'partner_id.property_account_position', 'partner_id.property_account_position.local')
    def _compute_local(self):
        if not self.partner_id or not self.partner_id.property_account_position:
            self.local = True
        else:
            self.local = self.partner_id.property_account_position.local

    def create_import_clearance(self, cr, uid, ids, context=None):
        clearance_obj = self.pool.get('import.clearance')
        picking_obj = self.pool.get('stock.picking')

        for order in self.browse(cr, uid, ids, context=context):

            # Chequeamos que no tengamos un despacho que aun no se avanzo
            # TODO: Por ahora chequeamos si esta en draft. Creo que deberiamos
            # tambien chequear si ya tiene el picking asociado.
            for clearance in order.clearance_ids:
                if clearance.state == 'draft':
                    raise exceptions.except_orm(_("Clearance Error"), _("You cannot create a new clearance while there is one in draft state"))

            bcra_pending = False
            # if order.payment_term_id.term_type in ['advanced', 'view']:
            #     bcra_pending = True

            # Creamos el import clearance a partir de este picking que se recibio
            vals = {
                'name': '',
                'origin': order.name,
                'purchase_id': order.id,
                'partner_id': order.partner_id.id,
                'bcra_pending': bcra_pending,
            }

            # Si tengo el picking, asociarlo. Ver en que estado esta
            # y si ya no esta asociado a otro clearance
            picking_id = None
            for pick in order.picking_ids:
                if not pick.clearance_id:
                    picking_id = pick.id
                    vals['picking_id'] = picking_id
                    break

            # Creamos el despacho de importacion
            clearance_id = clearance_obj.create(cr, uid, vals, context=context)

            if picking_id:
                picking_obj.write(cr, uid, picking_id, {'clearance_id': clearance_id}, context=context)

        return clearance_id

    def action_picking_create(self, cr, uid, ids, context=None):
        res = super(purchase_order, self).action_picking_create(cr, uid, ids, context=context)

        stock_picking_obj = self.pool.get('stock.picking')
        clearance_obj = self.pool.get('import.clearance')
        purchase_orders = self.browse(cr, uid, ids, context=context)

        # Intentamos asociar el Despacho al Picking que se esta creando,
        # eso si existe el Despacho en este momento y cumple con las condiciones
        for order in purchase_orders:
            if not order.local:
                clearance_id = None
                for clearance in order.clearance_ids:
                    # TODO: Ver que puede ser que este en otro estado
                    if clearance.state == 'draft' and not clearance.picking_id:
                        clearance_id = clearance.id
                        break

                if clearance_id:
                    stock_picking_obj.write(cr, uid, res, {'clearance_id': clearance_id, 'local': order.local}, context=context)
                    clearance_obj.write(cr, uid, clearance_id, {'picking_id': res}, context=context)
        return res

purchase_order()
