# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2013 E-MIPS (http://www.e-mips.com.ar) All Rights Reserved.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import _, fields, models


class stock_production_lot(models.Model):
    _name = 'stock.production.lot'
    _inherit = 'stock.production.lot'

    # TODO: Agregar chequeo de formato de este numero
    clearance_number = fields.Char('Clearance Number', size=32)


class stock_picking_forwarder(models.Model):
    _name = "stock.picking.forwarder"
    _description = "Picking Forwarder"

    name = fields.Char('Forwarder', size=32)


class stock_picking(models.Model):
    _name = 'stock.picking'
    _inherit = 'stock.picking'

    # TODO: Agregar chequeo de formato de este numero
    clearance_id = fields.Many2one('import.clearance', 'Clearance', readonly=True, ondelete='set null')
    incoterm_id = fields.Many2one('stock.incoterms', 'Incoterm',
                                  help="Incoterm which stands for 'International Commercial "
                                  "terms' implies its a series of sales terms which are used "
                                  "in the commercial transaction.")
    forwarder_id = fields.Many2one('stock.picking.forwarder', 'Forwarder')
    guide = fields.Char('Guide', size=32)
    expected_date = fields.Date('Expected date')
    country_arrival = fields.Date('Country arrival')
    date_shipper = fields.Date('Dispatcher notify')
    company_arrival = fields.Date('Company arrival')
    local = fields.Boolean('Local transaction', default=True, readonly=True)

    def do_partial(self, cr, uid, ids, partial_datas, context=None):
        clearance_obj = self.pool.get('import.clearance')
        #prodlot_obj = self.pool.get('stock.production.lot')
        res = super(stock_picking, self).do_partial(cr, uid, ids, partial_datas, context=context)

        for origpicking, picking_vals in res.iteritems():
            delivered_picking = self.browse(cr, uid, picking_vals['delivered_picking'], context=context)

            # Reasociamos al picking que se recibio
            clearance = delivered_picking.clearance_id
            if clearance:
                clearance_obj.write(cr, uid, clearance.id, {'picking_id': delivered_picking.id}, context=context)

            # Si son distintos, entonces hay que desaso
            if origpicking != delivered_picking.id:
                self.write(cr, uid, origpicking, {'clearance_id': False}, context=context)

            # TODO: Todavia falta ver el tema de los prod_lots en las facturas y el despacho
#            clearance_number = context.get('clearance_number', False)
#            if picking.type == 'in' and clearance_number:
#                print picking.name
#
#                for move_line in picking.move_lines:
#                    prodlot_obj.write(cr, uid, move_line.prodlot_id.id, {'clearance_number': clearance_number}, context=context)
#
#                # TODO: Manejar el caso que ya exista el clearance. no deberia pasar.
#                # Creamos el import clearance a partir de este picking que se recibio
#                vals = {
#                    'name': context.get('clearance_number', False),
#                    'origin': picking.origin,
#                    'date': picking.date_done,
#                    'date_arrival': picking.date_done,
#                    'picking_id': picking.id,
#                }
#
#                # Pasamos los adelantos al clearance creado
#                advancement_ids = [a.id for a in picking.advancement_detail_ids]
#                vals['advancement_detail_ids'] = [(6, 0, advancement_ids)]
#
#                # Creamos el despacho de importacion
#                clearance_id = clearance_obj.create(cr, uid, vals, context=context)
#                self.write(cr, uid, picking.id, {'clearance_id': clearance_id}, context=context)

        return res

    def _invoice_hook(self, cr, uid, picking, invoice_id):
        clearance_obj = self.pool.get('import.clearance')
        invoice_obj = self.pool.get('account.invoice')

        res = super(stock_picking, self)._invoice_hook(cr, uid, picking, invoice_id)

        # Chequeamos si viene de un Incoming Shipment y si tiene asignado un clearance
        if picking.type == 'in' and picking.clearance_id:
            clearance_obj.write(cr, uid, picking.clearance_id.id, {'invoice_ids': [(4, invoice_id)]})
        elif picking.type == 'out':
            clearance_numbers = []
            for move_line in picking.move_lines:
                number = move_line.prodlot_id and move_line.prodlot_id.clearance_number or False

                if number:
                    clearance_numbers.append(number)

            notes = _("Clearance Numbers: ") + ' '.join(clearance_numbers)
            # TODO: Por ahora se lo chantamos en comments. Lo deberiamos escribir en otro lado
            invoice_obj.write(cr, uid, [invoice_id], {'comment': notes})

        return res

    # TODO: Ver si esto es necesario en v8
#    def action_embarked(self, cr, uid, ids, context=None):
#        pick = self.browse(cr, uid, ids[0])
#
#        data = {'embarked': True}
#        if not pick.date_embarked:
#            date_embarked = time.strftime(DEFAULT_SERVER_DATETIME_FORMAT)
#            data.update({'date_embarked': date_embarked})
#        else:
#            date_embarked = pick.date_embarked
#
#        self.write(cr, uid, ids, data, context=context)
#
#        if pick.purchase_id:
#            self.pool.get('purchase.order').write(cr, uid, pick.purchase_id.id,
#                    {'date_embarked': date_embarked}, context=context)
#        return True
#
    # TODO: Ver si esto es necesario en v8 para parciales
    # TODO: Revisar recepciones parciales que no queden datos del embarque anterior
#    def copy(self, cr, uid, id, default=None, context=None):
#
#        default['incoterm_id'] = False
#        #default['forwarder_id'] = False
#        default['guide'] = ''
#        default['djai'] = ''
#        default['expected_date'] = False
#        default['embarked'] = False
#        default['date_embarked'] = False
#        default['country_arrival'] = False
#        default['date_shipper'] = False
#        default['company_arrival'] = False
#
#        res = super(stock_picking, self).copy(cr, uid, id, default, context)
#        return res
