# -*- coding: utf-8 -*-
##############################################################################
#
#    Service module for OpenERP, Open Source Management Solutions.
#    Copyright (C) 2015 E-MIPS/Eynes (<http://www.eynes.com.ar>). All rights
#    reserved.
#
#    This file is a part of Service
#
#    Service is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Service is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.addons import decimal_precision as dp
from openerp.osv import fields, orm
from openerp.tools import drop_view_if_exists
from openerp.tools.translate import _

class advancement_report(orm.Model):
    _auto = False
    _name = 'clearance.advancement.report'
    _description = 'Clearance Advancement Report'

    _columns = {
        'clearance_id': fields.many2one('import.clearance', 'Clearance', readonly=True),
        'account_id': fields.many2one('account.account', 'Account'),
        'debit': fields.float(digits_compute=dp.get_precision('Account'), string='Debit'),
        'credit': fields.float(digits_compute=dp.get_precision('Account'), string='Credit'),
        'balance': fields.float(digits_compute=dp.get_precision('Account'), string='Balance'),
    }

    def init(self, cr):
        """
        Use an SQL view
        """
        drop_view_if_exists(cr, 'clearance_advancement_report')
        # We use product_id+suitcase_id as id of the view to get an unique id that will be
        # consistent through the views.
        # The SELECT statement stored as out_moves doesn't filter stock_moves only in
        # 'done' state but also in 'assigned', 'draft', and 'confirmad'. This is
        # useful when a record is deleted from the kanban view.
        cr.execute("""
                   CREATE OR REPLACE VIEW clearance_advancement_report AS
                   (
SELECT row_number () over () as id, clearance_id, account_id, sum(debit) AS debit, sum(credit) AS credit, sum(balance) AS balance
    FROM (
        SELECT DISTINCT l.id, c.id as clearance_id, a.id AS account_id, l.debit as debit, l.credit as credit, l.debit-l.credit AS balance
        FROM import_clearance c JOIN clearance_move_rel rel ON rel.clearance_id=c.id
        JOIN account_move m ON rel.move_id=m.id
        JOIN account_move_line l ON l.move_id=m.id
        JOIN account_account a ON a.id=l.account_id
        JOIN import_advancement_type at ON at.account_id=a.id
) AS report
GROUP BY clearance_id, account_id
ORDER BY clearance_id
                   )
        """)

#    def fields_view_get(self, cr, uid, view_id, view_type="form", context=None, toolbar=False):
#        res = super(SuitcaseCurrentMaterials, self).fields_view_get(cr, uid, view_id, view_type,
#                                                                    context, toolbar)
#        if view_type == "search":
#            cm_obj = self.pool["suitcase.current.materials"]
#            cm_ids = cm_obj.search(cr, uid, [])
#            all_cm = cm_obj.browse(cr, uid, cm_ids)
#            dom = tuple(cm.product_id.id for cm in all_cm)
#            if dom:
#                res["fields"]["product_id"]["domain"].append(("id", "in", dom))
#        return res
