# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2015 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2015 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.tests.common import TransactionCase
from openerp import exceptions
from datetime import datetime
import time
import unittest2

# TODO: Testing de campo local, con compras locales
# TODO: Testing de embarques parciales
# TODO: Testing de marcado como embarcado

class TestClearance(TransactionCase):
    def setUp(self):
        super(TestClearance, self).setUp()

        self.product_product = self.env['product.product']
        self.purchase_order = self.env['purchase.order']
        self.purchase_order_line = self.env['purchase.order.line']
        self.pricelist = self.env['product.pricelist']
        self.clearance_model = self.env['import.clearance']
        self.payment_mode = self.env['account.journal']
        self.adv_detail_model = self.env['import.advancement.detail']
        self.invoice_model = self.env['account.invoice']
        self.expense_template_wiz = self.env['expense.template.use']
        self.move_model = self.env['account.move']

        self.currency_usd = self.env.ref('base.USD')
        self.currency_eur = self.env.ref('base.EUR')
        self.partner_id = self.env.ref('account_import_clearance.res_partner_supplier_exterior')
        self.partner_expense_id = self.env.ref('base.res_partner_3')
        self.product_pc1 = self.env.ref('product.product_product_3')
        self.product_expense_id = self.env.ref('product.product_product_2')
        pricelist_id = self.env.ref('product.list0')
        self.location_id = self.env.ref('stock.stock_location_stock')
        self.supplier_location_id = self.env.ref('stock.stock_location_suppliers')
        payment_term_id = self.env.ref('account.account_payment_term')
        self.account_cash_id = self.env.ref('account.cash')
        self.adv_vep_type_id = self.env.ref('account_import_clearance.advancement_type_vep')
        self.expense_template = self.env.ref('account_import_clearance.expense_template_1')
        self.fp_exterior = self.env.ref('account_import_clearance.fiscal_position_exterior')

        po_vals = {
            'partner_id': self.partner_id.id,
            'location_id': self.location_id.id,
            'pricelist_id': pricelist_id.id,
            # 'type': 'place',
            'fiscal_position': self.fp_exterior.id,
            'payment_term_id': payment_term_id.id,
            'local': False,
            'order_line': [
                (0, 0, {
                    'product_id': self.product_pc1.id,
                    'name': 'description',
                    'date_planned': time.strftime('%Y-%m-%d'),
                    'product_qty': 5,
                    'price_unit': 500.0,
                })],
        }
        self.po_01 = self.purchase_order.create(po_vals)

        # Lang
        self.env.user.lang = "en_US"

    def _create_clearance(self, po):
        po_01 = self.po_01
        self.assertFalse(po_01.local)
        self.assertEquals(po_01.state, 'draft')

        # La confirmamos, deberia quedar en approved
        po_01.signal_workflow('purchase_confirm')
        self.assertEquals(po_01.state, 'approved')

        # Creamos el clearance
        clearance_id = po_01.create_import_clearance()
        clearance = self.clearance_model.browse(clearance_id)
        return clearance

    def _add_advancement(self, clearance, adv_data):

        adv_type, amount = adv_data

        adv_detail_vals = {
            'name': 'An Advancement',
            'advancement_type_id': adv_type,
            'clearance_id': clearance.id,
        }

        adv_detail = self.adv_detail_model.create(adv_detail_vals)
        adv_detail.amount_total = amount
        adv_detail.onchange_advancement_type()
        return adv_detail

    def _add_expenses_by_template(self, clearance, template_id):

        context = {
            'active_id': clearance.id,
        }

        wiz_vals = {
            'template_id': template_id,
            'group_partners': False
        }
        res = self.expense_template_wiz.with_context(context).default_get(['broker_id'])
        wiz_vals.update(res)

        wiz_id = self.expense_template_wiz.create(wiz_vals)

        wiz_id.with_context({'active_id': clearance.id}).use_template()
        return

    def _pay_advancement(self, adv_detail):

        move = adv_detail.move_id
        journal_id = move.journal_id.id
        date_today = datetime.today().strftime('%Y-%m-%d')
        partner = adv_detail.advancement_type_id.partner_id

        # Lo pagamos en cash, creando un asiento manual ahora
        move_lines = [(0, 0, {
                    'name': "Test payment",
                    'debit': 0.0,
                    'credit': adv_detail.amount_total,
                    'account_id': self.account_cash_id.id,
                    'journal_id': journal_id,
                    'period_id': move.period_id.id,
                    'partner_id': partner.id,
                    'date': date_today}),
                (0, 0, {
                    'name': "Reconcile",
                    'debit': adv_detail.amount_total,
                    'credit': 0.0,
                    'account_id': partner.property_account_payable.id,
                    'journal_id': journal_id,
                    'period_id': move.period_id.id,
                    'partner_id': partner.id,
                    'date': date_today})]

        payment_move = self.move_model.create({
                'ref': "Pay Advancement",
                'line_id': move_lines,
                'journal_id': journal_id,
                'date': date_today, #clearance['date'],
                'company_id': adv_detail.clearance_id.company_id.id,
        })

        # Conciliamos las lineas
        lines0 = move.line_id.filtered(lambda x: x.account_id.type == 'payable')
        lines1 = payment_move.line_id.filtered(lambda x: x.account_id.type == 'payable')

        lines = lines0+lines1
        lines.reconcile()
        return payment_move

    def test_001_clearance_create(self):
        """Test creation of import clearance."""

        # Creamos el despacho a partir de la po_01
        clearance = self._create_clearance(self.po_01)

        # Chequeamos el picking & clearance
        self.assertTrue(clearance.picking_id.id)
        self.assertEquals(clearance.purchase_id.id, self.po_01.id)
        self.assertEquals(clearance.state, 'draft')

        # Excepcion por falta de informacion
        with self.assertRaises(exceptions.except_orm):
            clearance.action_confirm()

        clearance.write({'currency_id': self.currency_eur.id,
                 'name': 'ABC123'})

        # Para que tengamos el currency_rate
        clearance.onchange_currency_id()
        currency_rate = clearance.currency_rate

        # Excepcion por falta de informacion
        with self.assertRaises(exceptions.except_orm):
            clearance.action_confirm()

        # Cargamos un pago a cuenta por el VEP
        adv_detail = self._add_advancement(clearance, [self.adv_vep_type_id.id, 1000.0])

        with self.assertRaises(exceptions.except_orm):  # No hay asientos de adelantos creados
            clearance.action_confirm()

        # Vamos confirmando todo
        self.assertEquals(adv_detail.state, 'new')
        clearance.action_create_advancement()
        self.assertEquals(adv_detail.state, 'post')
        clearance.action_confirm()

        # Chequeamos que se tomo correctamente el adelanto
        self.assertEquals(adv_detail.amount_total, 1000.0)

        # Con esta accion se crean los asientos de adelantos & adv_detail is posted
        clearance.action_create_advancement()
        self.assertEquals(adv_detail.state, 'post')

        # Ready to confirm the clearance
        clearance.action_confirm()
        self.assertEquals(clearance.state, 'confirm')

        # Tiene que estar el asiento de adelanto
        self.assertEquals(len(clearance.account_move_ids), 1)

        # Antes de cargar gastos, intentamos postear el despacho, debe fallar
        with self.assertRaises(exceptions.except_orm):
            clearance.action_post()

        # Cargamos facturas de gastos
        self._add_expenses_by_template(clearance, self.expense_template.id)

        # Chequeamos facturas de gasto
        clearance.refresh()
        invoice = clearance.expense_invoice_ids[0]
        invoice.onchange_currency_id()
        invoice.write({'internal_number': '0001-00000010',
            'currency_rate': 3.0})
        invoice.button_reset_taxes()

        self.assertTrue(invoice.advancement_type_id)

        # Posteamos para crear los asientos contables
        # TODO: Chequeamos que los adelantos ya estén pagados???
        clearance.action_post()
        self.assertEquals(clearance.advancement_detail_ids[0].state, 'post')

        for payable in clearance.payable_grouped_ids:
            self.assertTrue(payable.advancement_type_id)

        # Cerramos sabiendo que no esta asociada la factura de proveedor
        with self.assertRaisesRegexp(exceptions.except_orm, 'At least one supplier invoice.*$'):
            clearance.action_pay_and_close()

        # Pagamos y conciliamos el adelanto
        self._pay_advancement(adv_detail)

        # Agregamos la factura de proveedor
        clearance.write({
            'invoice_ids': self.po_01.invoice_ids[0].id
        })

        # TODO: Este test fallaria porque quitamos la validacion de la factura del proveedor
        # a pedido de Rafalim
#        # Cerramos sabiendo que no esta validada la factura de proveedor
#        with self.assertRaisesRegexp(exceptions.except_orm, 'Supplier Invoice should be validated.*$'):
#            clearance.action_pay_and_close()

        sup_invoice = clearance.invoice_ids[0]
        sup_invoice.write({'internal_number': '0000111232'})
        sup_invoice.signal_workflow('invoice_open')
        self.assertEquals(sup_invoice.state, 'open')

        # Hacemos el ultimo paso de cerrar y conciliar pagos
        clearance.action_pay_and_close()
        self.assertEquals(clearance.state, 'closed')

#        # TODO: Este chequeo no funciona porque comentamos la parte de la validacion
#        # de la
#        self.assertEquals(len(clearance.account_move_ids), 4)

    TODO: Hacer un test por los tipos de cambio

    @unittest2.skip("Fix test. Not using check anymore.")
    def test_002_clearance_advancement_check(self):
        """Test creation of import clearance and advancement payment with check."""

        po_01 = self.po_01
        self.assertFalse(po_01.local)
        self.assertEquals(po_01.state, 'draft')

        # La confirmamos, deberia quedar en approved
        po_01.signal_workflow('purchase_confirm')
        self.assertEquals(po_01.state, 'approved')

        # Creamos el clearance
        clearance_id = po_01.create_import_clearance()
        clearance = self.clearance_model.browse(clearance_id)

        self.assertTrue(clearance.picking_id.id)
        self.assertEquals(clearance.purchase_id.id, self.po_01.id)
        self.assertEquals(clearance.state, 'draft')

        clearance.write({'name': 'ABC123'})
        # Cargamos un pago a cuenta por el VEP
        adv_detail_vals = {
            'name': 'VEP advancement',
            'advancement_type_id': self.adv_vep_type_id.id,
            'clearance_id': clearance.id,
            'issued_check_ids': [
                (0, 0, {
                    'number': '998',
                    'amount': 1000,
                    'bank_id': self.env.ref('base.res_bank_1').id,
                    'account_bank_id': self.company_bank_account_id.id}),
            ]
        }
        adv_detail = self.adv_detail_model.create(adv_detail_vals)
        adv_detail.payment_line_ids[0].amount = 1000
        self.assertEquals(adv_detail.amount_total, 2000.0)

        # Now cash is cero
        adv_detail.payment_line_ids[0].amount = 0
        self.assertEquals(adv_detail.amount_total, 1000.0)

        with self.assertRaises(exceptions.except_orm):  # No hay asientos de adelantos creados, no se puede confirmar
            clearance.action_confirm()

        # Con esta accion se crean los asientos de adelantos & adv_detail is posted
        clearance.action_create_advancement()
        self.assertEquals(adv_detail.state, 'post')
        self.assertEquals(clearance.state, 'draft')

        # Ahora si confirmamos
        clearance.action_confirm()
        self.assertEquals(clearance.state, 'confirm')
        self.assertEquals(len(adv_detail.issued_check_ids), 1)
        issued_check = adv_detail.issued_check_ids[0]
        self.assertEquals(issued_check.state, 'issued')

        # Tiene que estar el asiento de adelanto
        self.assertEquals(len(clearance.account_move_ids), 1)


    def test_003_clearance_freight_and_insurance(self):
        """Test values of freight and insurance in different currency as currency company"""

        clearance = self._create_clearance(self.po_01)

        # Seteamos la currency en dolares
        clearance.write({'currency_id': self.currency_usd.id,
            'name': 'ABC123'})

        # Para que tengamos el currency_rate
        clearance.onchange_currency_id()
        currency_rate = clearance.currency_rate

        # Agregamos un adelanto
        adv_detail = self._add_advancement(clearance, [self.adv_vep_type_id.id, 0.0])

        # Vamos confirmando todo
        clearance.action_create_advancement()
        self.assertEquals(adv_detail.state, 'post')
        clearance.action_confirm()

        # Cargamos facturas de gastos
        self._add_expenses_by_template(clearance, self.expense_template.id)

        # Chequeamos facturas de gasto
        clearance.refresh()
        invoice = clearance.expense_invoice_ids[0]
        invoice.onchange_currency_id()
        invoice.write({'internal_number': '0001-00000010',
            'currency_rate': 3.0})
        invoice.button_reset_taxes()

        # Obtenemos el campo expense
        expense = clearance.expense

        # Cargamos valores de flete y seguro
        clearance.write({'fob': 1000, 'freight': 100, 'insurance': 200})

        # Obtenemos el campo expense_percent
        clearance.refresh()
        expense_percent = (clearance.expense + (clearance.freight+clearance.insurance)* \
                currency_rate) / (clearance.fob*currency_rate) * 100
        self.assertEquals(expense_percent, clearance.expense_percent)
