# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2014 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2014 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import models, fields, api, exceptions, _


class expense_template_use(models.TransientModel):
    _name = "expense.template.use"
    _description = "Expense Template Use"

    broker_id = fields.Many2one('res.partner', 'Partner')
    template_id = fields.Many2one('clearance.expense.template', 'Template', required=True)
    #'group_partners': fields.boolean('Group Partners', help="Check this f you want to group by partners in the same expense invoice"),

    def default_get(self, cr, uid, fields, context=None):
        res = super(expense_template_use, self).default_get(
                cr, uid, fields, context=context)

        if context is None:
            context = {}

        clearance_id = context.get('active_id', [])

        if 'broker_id' in fields:
            clearance = self.pool.get('import.clearance').browse(
                    cr, uid, clearance_id, context=context)

            res['broker_id'] = clearance.broker_id.id

        return res

    def _prepare_inv_line(self, cr, uid, account_id, line, context=None):
        """Collects require data from purchase order line that is used to create invoice line
        for that purchase order line
        :param account_id: Expense account of the product of PO line if any.
        :param browse_record line: Purchase order line browse record
        :return: Value for fields of invoice lines.
        :rtype: dict
        """
        return {
            'name': line.name,
            'account_id': account_id,
            'price_unit': line.product_id.standard_price or 0.0,
            'quantity': 1,
            'product_id': line.product_id.id,
            'uos_id': line.product_id.uom_id.id or False,
            #'invoice_line_tax_id': [(6, 0, [x.id for x in line.taxes_id])],
            #'account_analytic_id': line.account_analytic_id.id or False,
        }

    def use_template(self, cr, uid, ids, context=None):
        clearance_obj = self.pool.get("import.clearance")
        invoice_obj = self.pool.get('account.invoice')
        invoice_line_obj = self.pool.get('account.invoice.line')
        account_jrnl_obj = self.pool.get('account.journal')
        user = self.pool.get('res.users').browse(cr, uid, uid)
        if context is None:
            context={}

        # Valores del wizard
        wiz = self.browse(cr, uid, ids[0], context=context)
        template = wiz.template_id

        # Clearance
        clearance_id = context['active_id']
        clearance = clearance_obj.browse(cr, uid, clearance_id, context=context)

        if clearance.state == 'post':
            raise exceptions.except_orm(_("Clearance Error!"), _("You cannot add more expense invoices to this clearance!"))

        # Creamos las facturas correspondiente por cada linea de gastos
        invoices = {}

        for line in template.expense_line_ids:
            # Agrupamos por partner
            if not line.partner_id.id in invoices:
                invoices[line.partner_id.id] = []

            acc_id = line.expense_account_id.id
            inv_line_data = self._prepare_inv_line(cr, uid, acc_id, line, context=context)

            line_onchange_res = invoice_line_obj.product_id_change(cr, uid, [], inv_line_data['product_id'],
                                    inv_line_data['uos_id'], qty=1, name=inv_line_data['name'],
                                    type='in_invoice', partner_id=line.partner_id.id,
                                    fposition_id=line.partner_id.property_account_position.id,
                                    price_unit=inv_line_data['price_unit'], currency_id=False,
                                    context=context, company_id=False)

            origin = clearance.name or '' + '[' + clearance.origin + ']'
            inv_line_data.update({'origin': origin})
            inv_line_data.update(line_onchange_res['value'])
            iltaxes = [(4, t) for t in inv_line_data['invoice_line_tax_id']]
            inv_line_data['invoice_line_tax_id'] = iltaxes
            inv_id = invoice_line_obj.create(cr, uid, inv_line_data, context=context)
            invoices[line.partner_id.id].append((line,inv_id))

        def make_invoice_by_partner(partner, lines, lines_ids):
            """
                create a new invoice for one supplier
                @param partner : The object partner
                @param orders : The set of orders to add in the invoice
                @param lines : The list of line's id
            """
            name = lines and lines[0].name or ''
            journal_id = clearance.journal_id.id or False
            currency_company = user.company_id.currency_id.id
            if not journal_id:
                journal_id = account_jrnl_obj.search(cr, uid, [('type', '=', 'purchase')], context=None)
                journal_id = journal_id and journal_id[0] or False

            a = lines[0].account_id.id
            inv = {
                'name': name,
                'origin': origin,
                'type': 'in_invoice',
                'journal_id':journal_id,
                'reference' : partner.ref,
                'account_id': a,
                'partner_id': partner.id,
                'invoice_line': [(6,0,lines_ids)],
                'currency_id' : lines[0].in_clearance_currency and clearance.currency_id.id or currency_company,
                'clearance_id' : clearance.id,
                'clearance_taxable' : lines[0].clearance_taxable,
                'advancement_type_id' : lines[0].advancement_type_id.id,
                #'comment': multiple_order_invoice_notes(lines),
                'payment_term': lines[0].partner_id.property_payment_term.id,
                'fiscal_position': partner.property_account_position.id
            }

            inv_change = invoice_obj.onchange_partner_id(
                    cr, uid, [], inv['type'], inv['partner_id'],
                    date_invoice=False, payment_term=False, partner_bank_id=False,
                    company_id=False)

            inv.update(inv_change['value'])
            inv_id = invoice_obj.create(cr, uid, inv, context=context)
            invoice = invoice_obj.browse(cr, uid, inv_id, context=context)
            invoice.onchange_currency_id()
            invoice_obj.button_reset_taxes(cr, uid, [inv_id], context=context)
            return inv_id

        res = []
        for result in invoices.values():
            il = map(lambda x: x[1], result)
            lines = list(set(map(lambda x : x[0], result)))

            res.append(make_invoice_by_partner(lines[0].partner_id, lines, il))

#        return {
#            'domain': "[('id','in', ["+','.join(map(str,res))+"])]",
#            'name': _('Supplier Invoices'),
#            'view_type': 'form',
#            'view_mode': 'tree,form',
#            'res_model': 'account.invoice',
#            'view_id': False,
#            'context': "{'type':'in_invoice', 'journal_type': 'purchase'}",
#            'type': 'ir.actions.act_window'
#        }

        return {'type': 'ir.actions.act_window_close'}

expense_template_use()
